@echo off
REM Wrapper to run bcs-mcp-atlassian from this project (for Cursor MCP with local code).
REM Use this script as "command" in Cursor MCP config when workspace is not bcs-mcp-atlassian-bcs.
cd /d "%~dp0"
uv run bcs-mcp-atlassian %*
